"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BigIfElseRefactorer = void 0;
const ts_morph_1 = require("ts-morph");
const refactorer_model_1 = require("../models/refactorer.model");
const refactorer_utils_1 = require("../utils/refactorer.utils");
class BigIfElseRefactorer extends refactorer_model_1.Refactorer {
    constructor() {
        super(...arguments);
        this.REFACTORED_NODE_KIND = ts_morph_1.SyntaxKind.MethodDeclaration;
    }
    refactorNeeded(node) {
        const IF_STATEMENTS = node.getDescendantsOfKind(ts_morph_1.SyntaxKind.IfStatement);
        return IF_STATEMENTS.some((i) => {
            const BLOCKS = i.getDescendantsOfKind(ts_morph_1.SyntaxKind.Block);
            return BLOCKS.some((b) => b.compilerNode.statements.length > 5);
        });
    }
    /**
     * Copy current method then transform the copy to get refctored method
     * Put refactored method on current method object
     * @param method the current method
     * @returns {void}
     */
    refactor(node) {
        let methods = [];
        let parameters = [];
        let inputs = [];
        const NODE = node.transform((traversal) => {
            let currentNode = refactorer_model_1.Refactorer.wrapCurrentNode(node, traversal);
            this.catchInputs(currentNode, inputs);
            if (this.isConditionnedBlock(currentNode)) {
                currentNode.getDescendantsOfKind(ts_morph_1.SyntaxKind.VariableStatement).forEach((s) => {
                    const IDENTIFIER = s.getFirstDescendantByKind(ts_morph_1.SyntaxKind.Identifier).getFullText();
                    inputs = inputs.filter((i) => i.identifier !== IDENTIFIER);
                });
                this.keepOnlyParameters(currentNode, inputs, parameters);
                const METHOD_NAME = `methodToRename${methods.length}`;
                const PARAMETERS = parameters.map(({ identifier, type }) => refactorer_utils_1.RefactorerUtils.createSimpleParameter(identifier, type));
                const NEW_METHOD = refactorer_utils_1.RefactorerUtils.createSimpleMethod(METHOD_NAME, currentNode.compilerNode, PARAMETERS);
                methods.push(NEW_METHOD);
                const CONTAIN_RETURN = currentNode.compilerNode.statements.find((s) => s.kind === ts_morph_1.ts.SyntaxKind.ReturnStatement);
                const METHOD_CALL = refactorer_utils_1.RefactorerUtils.createMethodCall(METHOD_NAME, []);
                return ts_morph_1.ts.createBlock(CONTAIN_RETURN ? [ts_morph_1.ts.createReturn(METHOD_CALL)] : [METHOD_CALL]);
            }
            return currentNode.compilerNode;
        });
        this.addMethodToClass(NODE, methods);
        return NODE;
    }
    addMethodToClass(node, methods) {
        this.addTransformer({
            baseNode: node,
            nodeMethod: 'getParent',
            transformer: (traversal) => {
                const NODE = traversal.visitChildren();
                if (ts_morph_1.ts.isClassDeclaration(NODE)) {
                    const METHODS = ts_morph_1.ts.createNodeArray([...NODE.members, ...methods]);
                    return ts_morph_1.ts.createClassDeclaration([], [], NODE.name, [], [], METHODS);
                }
                return NODE;
            },
        });
    }
    keepOnlyParameters(node, inputs, parameters) {
        node.getDescendantsOfKind(ts_morph_1.SyntaxKind.Identifier).forEach((d) => {
            inputs.forEach((i) => {
                if (i.identifier === d.getFullText() && !parameters.includes(i))
                    parameters.push(i);
            });
        });
    }
    catchInputs(node, inputs) {
        if (this.isInputs(node)) {
            const IDENTIFIER = node.getFirstDescendantByKind(ts_morph_1.SyntaxKind.Identifier).getFullText();
            const TYPE = node === null || node === void 0 ? void 0 : node.compilerNode.type;
            if (!inputs.find((i) => i.identifier === IDENTIFIER))
                inputs.push({ identifier: IDENTIFIER, type: TYPE, isParameter: false });
        }
    }
    isConditionnedBlock(node) {
        return node.getParent() && ts_morph_1.Node.isIfStatement(node.getParent()) && ts_morph_1.Node.isBlock(node) && node.getStatements().length > 5;
    }
    isInputs(node) {
        return (ts_morph_1.Node.isParameterDeclaration(node) && !ts_morph_1.Node.isArrowFunction(node.getParent())) || ts_morph_1.Node.isVariableDeclaration(node);
    }
}
exports.BigIfElseRefactorer = BigIfElseRefactorer;
