"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TernaryToNullishCoalescing = void 0;
const ts_morph_1 = require("ts-morph");
const refactorer_model_1 = require("../models/refactorer.model");
class TernaryToNullishCoalescing extends refactorer_model_1.Refactorer {
    constructor() {
        super(...arguments);
        this.REFACTORED_NODE_KIND = ts_morph_1.SyntaxKind.MethodDeclaration;
    }
    refactorNeeded(node) {
        let refactorNeeded = false;
        const TERNARY_EXPRESSIONS = node.getDescendantsOfKind(ts_morph_1.SyntaxKind.ConditionalExpression);
        TERNARY_EXPRESSIONS.forEach((ternary) => {
            const COUNT = ternary.getChildCount();
            if (ts_morph_1.Node.isConditionalExpression(ternary) && COUNT > 0) {
                const FIRST_MEMBER = ternary.getChildAtIndex(0);
                const SECOND_MEMBER = ternary.getChildAtIndex(2);
                if (FIRST_MEMBER.getText() === SECOND_MEMBER.getText() && FIRST_MEMBER.getKind() === SECOND_MEMBER.getKind()) {
                    refactorNeeded = true;
                }
            }
        });
        return refactorNeeded;
    }
    /**
     * Copy current method then transform the copy to get refctored method
     * Put refactored method on current method object
     * @param method the current method
     * @returns {void}
     */
    refactor(node) {
        return node.transform((traversal) => {
            const currentNode = refactorer_model_1.Refactorer.wrapCurrentNode(node, traversal);
            if (ts_morph_1.Node.isConditionalExpression(currentNode)) {
                const FIRST_MEMBER = currentNode.getChildAtIndex(0);
                const SECOND_MEMBER = currentNode.getChildAtIndex(2);
                if (FIRST_MEMBER.getText() === SECOND_MEMBER.getText() && FIRST_MEMBER.getKind() === SECOND_MEMBER.getKind()) {
                    const THIRD_MEMBER = currentNode.getChildAtIndex(4);
                    return ts_morph_1.ts.createBinary(FIRST_MEMBER.compilerNode, ts_morph_1.SyntaxKind.QuestionQuestionToken, THIRD_MEMBER.compilerNode);
                }
            }
            return currentNode.compilerNode;
        });
    }
}
exports.TernaryToNullishCoalescing = TernaryToNullishCoalescing;
