"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UselessElseRefactorer = void 0;
const ts_morph_1 = require("ts-morph");
const refactorer_model_1 = require("../models/refactorer.model");
class UselessElseRefactorer extends refactorer_model_1.Refactorer {
    constructor() {
        super(...arguments);
        this.REFACTORED_NODE_KIND = ts_morph_1.SyntaxKind.MethodDeclaration;
    }
    /**
     * Check method structure to know if it needs refacto
     * if true refactor the method
     * @returns {void}
     * @param node
     */
    refactorNeeded(node) {
        var _a, _b;
        const FIRST_BLOCK = node.getFirstChildByKind(ts_morph_1.SyntaxKind.Block);
        const IF_STATEMENT = FIRST_BLOCK === null || FIRST_BLOCK === void 0 ? void 0 : FIRST_BLOCK.getChildrenOfKind(ts_morph_1.SyntaxKind.IfStatement)[0];
        const HAS_ELSE_STATEMENT = ((_a = IF_STATEMENT === null || IF_STATEMENT === void 0 ? void 0 : IF_STATEMENT.getChildrenOfKind(ts_morph_1.SyntaxKind.Block)) === null || _a === void 0 ? void 0 : _a.length) === 2;
        const HAS_RETURN_ON_IF = (_b = IF_STATEMENT === null || IF_STATEMENT === void 0 ? void 0 : IF_STATEMENT.getFirstChildByKind(ts_morph_1.SyntaxKind.Block)) === null || _b === void 0 ? void 0 : _b.getFirstChildByKind(ts_morph_1.SyntaxKind.ReturnStatement);
        return Boolean(IF_STATEMENT && HAS_ELSE_STATEMENT && HAS_RETURN_ON_IF);
    }
    /**
     * Copy current method then transform the copy to get refctored method
     * Put refactored method on current method object
     * @returns {void}
     * @param node
     */
    refactor(node) {
        var _a;
        let elseStatements = [];
        node.transform((traversal) => {
            const currentNode = refactorer_model_1.Refactorer.wrapCurrentNode(node, traversal);
            if (ts_morph_1.Node.isIfStatement(currentNode) && currentNode.getElseStatement()) {
                elseStatements = currentNode.getElseStatement().getChildren().map(s => s.getFullText());
                elseStatements = elseStatements.slice(1, elseStatements.length - 1);
                if (elseStatements[0])
                    elseStatements[0] = elseStatements[0].replace(/\n/, '');
                return ts_morph_1.ts.createIf(currentNode.getExpression().compilerNode, currentNode.getThenStatement().compilerNode);
            }
            return currentNode.compilerNode;
        });
        (_a = node.getFirstDescendantByKind(ts_morph_1.SyntaxKind.Block)) === null || _a === void 0 ? void 0 : _a.addStatements(elseStatements);
        return node;
    }
}
exports.UselessElseRefactorer = UselessElseRefactorer;
