"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectService = void 0;
const ts_morph_1 = require("ts-morph");
class ProjectService {
    constructor(tsConfigFilePath) {
        this.refactorProposals = [];
        this.project = new ts_morph_1.Project({ tsConfigFilePath });
    }
    /**
     * Add new refactor proposals to the list, and replace the existing ones
     * @param refactorProposals
     */
    addToRefactorProposals(refactorProposals) {
        refactorProposals.forEach(proposal => {
            const index = this.refactorProposals.findIndex(r => r.id === proposal.id);
            if (index !== -1) {
                this.refactorProposals[index] = proposal;
            }
            else {
                this.refactorProposals.push(proposal);
            }
        });
    }
    /**
     * Get systems of a given Syntax Kind
     * @param kind the kind
     * @returns {Node[]}
     */
    getNodesOfKinds(kind) {
        const SYSTEMS = [];
        this.project.getSourceFiles().forEach((sf) => {
            const FILE_SYSTEMS = sf.getDescendantsOfKind(kind);
            SYSTEMS.push(...FILE_SYSTEMS);
        });
        return SYSTEMS;
    }
}
exports.ProjectService = ProjectService;
