"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RefactorReportService = void 0;
const eol = require("eol");
const fs = require("fs-extra");
const Handlebars = require("handlebars");
const options_model_1 = require("../../core/models/options.model");
const file_service_1 = require("../../core/services/file.service");
/**
 * Service generating files reports
 */
class RefactorReportService {
    constructor(systems, astFolder) {
        this.systems = systems;
        this.astFolder = astFolder;
    }
    get refactorProposals() {
        return this.systems;
    }
    /**
     * Register partials and compile template
     * @returns {void}
     */
    generateRefactorReport() {
        this.registerPartial('refactorProposals', 'refactor-proposals');
        this.registerPartial('refactorComparaison', 'refactor-comparaison');
        this.registerPartial('methodCode', 'method-script');
        const TEMPLATE_PATH = `${options_model_1.Options.pathGeneseNodeJs}/automatic-refactoring/templates/handlebars/refactor-proposals.handlebars`;
        const REPORT_TEMPLATE = this.getFileFromPath(TEMPLATE_PATH);
        this.template = Handlebars.compile(REPORT_TEMPLATE);
        this.writeRefactorReport();
    }
    /**
     * Generate refactor report HTML
     * @returns {void}
     */
    writeRefactorReport() {
        var _a, _b;
        const RELATIVE_ROOT = file_service_1.getRouteToRoot((_a = this.astFolder) === null || _a === void 0 ? void 0 : _a.relativePath);
        const TEMPLATE = this.template({ proposals: this.refactorProposals, relativeRoot: RELATIVE_ROOT });
        const RELATIVE_PATH = file_service_1.constructLink((_b = this.astFolder) === null || _b === void 0 ? void 0 : _b.relativePath);
        const OUT_DIR = file_service_1.constructLink(options_model_1.Options.pathOutDir);
        const PATH_REPORT = `${file_service_1.deleteLastSlash(OUT_DIR)}/${file_service_1.deleteLastSlash(RELATIVE_PATH)}/refactor-report.html`;
        fs.outputFileSync(PATH_REPORT, TEMPLATE, { encoding: 'utf-8' });
    }
    /**
     * Registers a HandleBar's partial for a given partial's name and a given filename
     * @param partialName   // The name of the partial
     * @param filename      // The name of the file
     * @returns {void}
     */
    registerPartial(partialName, filename) {
        const PARTIAL_PATH = `${options_model_1.Options.pathGeneseNodeJs}/automatic-refactoring/templates/handlebars/${filename}.handlebars`;
        const PARTIAL = this.getFileFromPath(PARTIAL_PATH);
        Handlebars.registerPartial(partialName, PARTIAL);
    }
    /**
     * read file, normalize content and return it
     * @param path the file path
     * @returns {string}
     */
    getFileFromPath(path) {
        const FILE_CONTENT = fs.readFileSync(path, 'utf-8');
        return eol.auto(FILE_CONTENT);
    }
}
exports.RefactorReportService = RefactorReportService;
