"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonAstToReports = void 0;
const init_service_1 = require("./services/init.service");
const reports_service_1 = require("./services/report/reports.service");
const terminalLink = require("terminal-link");
/**
 * Main process jsonAst analysis and reports
 */
class JsonAstToReports {
    /**
     * Starts the analysis
     * @param pathCommand
     * @param jsonAstPath
     * @param markdown
     * @param consoleMode
     */
    static start(pathCommand, jsonAstPath = '/json-ast.json', markdown, consoleMode) {
        let result = undefined;
        const jsonAst = new init_service_1.InitService().generateAllFromJsonAst(JsonAstToReports.getJsonAst(pathCommand + jsonAstPath));
        jsonAst.astFolder.evaluate();
        if (markdown) {
            reports_service_1.ReportsService.generateMarkdownReports(jsonAst);
        }
        else if (consoleMode) {
            result = reports_service_1.ReportsService.generateConsoleReports(jsonAst);
        }
        else {
            reports_service_1.ReportsService.generateAllReports(jsonAst);
            const link = terminalLink('folder-report.html', `file://${pathCommand}/genese/complexity/reports/folder-report.html`);
            result = `Please open in your browser the file "${link}" located in your genese reports folder.`;
        }
        this.astFolder = jsonAst.astFolder;
        return result;
    }
    /**
     * Get the complexity value of the given JsonAst
     * @param jsonAst
     * @returns {number}
     */
    static getTotalCpx(jsonAst) {
        const json = new init_service_1.InitService().generateAllFromJsonAst(jsonAst);
        json.astFolder.evaluateStandalone();
        return json.astFolder.stats.totalCognitiveComplexity;
    }
    /**
     * Returns the content of the JsonAst file
     * @param jsonAstPath
     */
    static getJsonAst(jsonAstPath) {
        return require(jsonAstPath);
    }
}
exports.JsonAstToReports = JsonAstToReports;
