"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AstFolderConsoleReportService = void 0;
const ast_folder_service_1 = require("../ast/ast-folder.service");
const options_model_1 = require("../../../core/models/options.model");
const ast_method_service_1 = require("../ast/ast-method.service");
/**
 * Service generating folders reports
 */
class AstFolderConsoleReportService {
    constructor(astFolder) {
        this.astFolder = undefined; // The AstFolder relative to this service
        this.astFolderService = new ast_folder_service_1.AstFolderService(); // The service relative to AstFolders
        this.methodsArrayReport = []; // The array of methods reports
        this.template = undefined; // The HandleBar template used to generate the report
        this.astFolder = astFolder;
        this.astFolderService.astFolder = this.astFolder;
    }
    /**
     * Generates the folder's report
     */
    generateReport() {
        this.setMethodsArraySortedByDecreasingCognitiveCpx(this.astFolder);
        return this.methodsArrayReport;
        // this.writeReport();
    }
    /**
     * Set the array of methods sorted by decreasing cognitive complexity
     * @param astFolder    // The AstFolder to analyse
     */
    setMethodsArraySortedByDecreasingCognitiveCpx(astFolder) {
        this.setTsFileReport(astFolder);
        this.setMethodsArrayReport(astFolder);
        this.methodsArrayReport = ast_method_service_1.AstMethodService.sortByDecreasingCognitiveCpx(this.methodsArrayReport);
        this.methodsArrayReport = this.methodsArrayReport.filter(e => e.cpxIndex >= options_model_1.Options.getThresholds().cognitive.warning);
    }
    /**
     * Recursion setting the array of methods reports of each subFolder
     * @param astFolder    // The AstFolder to analyse
     */
    setMethodsArrayReport(astFolder) {
        for (const subFolder of astFolder.children) {
            this.setTsFileReport(subFolder);
            this.setMethodsArrayReport(subFolder);
        }
    }
    /**
     * Recursion setting the array of methods reports of each subFolder's files
     * @param astFolder    // The AstFolder to analyse
     */
    setTsFileReport(astFolder) {
        for (const tsFile of astFolder.astFiles) {
            this.setAstMethodReport(tsFile);
        }
    }
    /**
     * Recursion setting the array of methods reports of each file's methods
     * @param astFile    // The AstFile to analyse
     */
    setAstMethodReport(astFile) {
        for (const astMethod of astFile.astMethods) {
            this.methodsArrayReport.push({
                cognitiveColor: astMethod.cognitiveStatus.toLowerCase(),
                cpxIndex: astMethod.cpxIndex,
                cyclomaticColor: astMethod.cyclomaticStatus.toLowerCase(),
                cyclomaticValue: astMethod.cyclomaticCpx,
                filename: `file://${astFile.astFolder.path}/${astFile.name}`,
                linkFile: `${astFile.astFolder.path}/${astFile.name}`,
                methodName: astMethod.name,
            });
        }
    }
    /**
     * Fills the HandleBar's template
     */
    writeReport() {
        console.table(this.methodsArrayReport, ['file', 'methodName', 'cpxIndex']);
    }
}
exports.AstFolderConsoleReportService = AstFolderConsoleReportService;
