"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const cst_to_ast_1 = require("../cst-to-ast");
const bin_ops_precedence_1 = require("../const/bin-ops-precedence");
// @ts-ignore
function run(cstNode, children) {
    const unaryExpressions = children.unaryExpression;
    const binaryOperators = children.BinaryOperator;
    const less = children.Less;
    const greater = children.Greater;
    const assignmentOperator = children.AssignmentOperator;
    const unaryExpressionsAst = [...[].concat(...unaryExpressions.map(e => cst_to_ast_1.cstToAst(e)))];
    if (binaryOperators || less || greater) {
        return binaryOperatorsCase(binaryOperators, less, greater, unaryExpressionsAst);
    }
    else if (assignmentOperator) {
        return assignmentOperatorCase(cstNode, children, unaryExpressionsAst, assignmentOperator);
    }
    else {
        return [
            ...unaryExpressionsAst,
        ];
    }
}
exports.run = run;
function binaryOperatorsCase(binaryOperators, less, greater, unaryExpressionsAst) {
    let binaryOperatorsAst = constructBinaryOperatorsAst(binaryOperators, less, greater);
    const alternate = [];
    for (let i = 0; i < binaryOperatorsAst.length; i++) {
        alternate.push(unaryExpressionsAst[i], binaryOperatorsAst[i]);
    }
    alternate.push(...unaryExpressionsAst.slice(binaryOperatorsAst.length), ...binaryOperatorsAst.slice(binaryOperatorsAst.length));
    const separatedExps = splitExpression(alternate);
    return toBinaryExpression(separatedExps.op, separatedExps.left, separatedExps.right);
}
function constructBinaryOperatorsAst(binaryOperators, less, greater) {
    var _a;
    let binaryOperatorsAst = (_a = binaryOperators === null || binaryOperators === void 0 ? void 0 : binaryOperators.map(e => cst_to_ast_1.cstToAst(e, 'binaryOperator'))) !== null && _a !== void 0 ? _a : [];
    const lessAndGreaterAst = [];
    lessAndGreaterAst.push(...reconstructOperators(less));
    lessAndGreaterAst.push(...reconstructOperators(greater));
    lessAndGreaterAst.forEach(op => {
        binaryOperatorsAst.push({
            kind: cst_to_ast_1.getBinaryOperatorName(op.map(e => e.image).join('')),
            start: op[0].startOffset,
            end: op[op.length - 1].endOffset,
            pos: op[0].startOffset
        });
    });
    binaryOperatorsAst = binaryOperatorsAst.sort((a, b) => {
        return a.start - b.start;
    });
    return binaryOperatorsAst;
}
function assignmentOperatorCase(cstNode, children, unaryExpressionsAst, assignmentOperator) {
    var _a, _b;
    const expression = children.expression;
    return {
        kind: 'BinaryExpression',
        start: cstNode.location.startOffset,
        end: cstNode.location.endOffset,
        pos: cstNode.location.startOffset,
        children: [
            ...unaryExpressionsAst,
            ...(_a = assignmentOperator === null || assignmentOperator === void 0 ? void 0 : assignmentOperator.map(e => cst_to_ast_1.cstToAst(e, 'assignmentOperator'))) !== null && _a !== void 0 ? _a : [],
            ...[].concat(...(_b = expression === null || expression === void 0 ? void 0 : expression.map(e => cst_to_ast_1.cstToAst(e))) !== null && _b !== void 0 ? _b : [])
        ]
    };
}
function reconstructOperators(elements) {
    if (!elements || elements.length === 0)
        return [];
    const result = [];
    const indexes = elements.map((e, i) => {
        var _a;
        return (e === null || e === void 0 ? void 0 : e.startOffset) + 1 === ((_a = elements[i + 1]) === null || _a === void 0 ? void 0 : _a.startOffset) ? null : i + 1;
    }).filter(e => e);
    indexes.forEach((i, index) => {
        var _a;
        result.push(elements.slice((_a = indexes[index - 1]) !== null && _a !== void 0 ? _a : 0, i));
    });
    return result;
}
function splitExpression(list) {
    if (list.length === 1) {
        return list[0];
    }
    const result = {
        op: undefined,
        left: undefined,
        right: undefined
    };
    bin_ops_precedence_1.binaryOperatorsPrecedence.forEach(ops => {
        const index = list.findIndex(e => ops.includes(e.kind));
        if (index !== -1 && !result.op) {
            result.op = list[index];
            result.left = splitExpression(list.slice(0, index));
            result.right = splitExpression(list.slice(index + 1, list.length + 1));
        }
    });
    return result;
}
function toBinaryExpression(op, left, right) {
    const children = [
        left.op ? toBinaryExpression(left.op, left.left, left.right) : left,
        op,
        right.op ? toBinaryExpression(right.op, right.left, right.right) : right,
    ];
    let mostLeft = left;
    while (mostLeft.left) {
        mostLeft = mostLeft.left;
    }
    let mostRight = right;
    while (mostRight.right) {
        mostRight = mostRight.right;
    }
    return {
        kind: 'BinaryExpression',
        start: mostLeft.start,
        end: mostRight.end,
        pos: mostLeft.pos,
        children: children
    };
}
