"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const cst_to_ast_1 = require("../cst-to-ast");
const genese_mapper_1 = require("genese-mapper");
function run(cstNode, children) {
    var _a, _b;
    const primaryPrefix = children.primaryPrefix;
    const primarySuffix = children.primarySuffix;
    const primaryPrefixAst = [].concat(...(_a = primaryPrefix === null || primaryPrefix === void 0 ? void 0 : primaryPrefix.map(e => cst_to_ast_1.cstToAst(e))) !== null && _a !== void 0 ? _a : []);
    const primarySuffixAst = [].concat(...(_b = primarySuffix === null || primarySuffix === void 0 ? void 0 : primarySuffix.map(e => cst_to_ast_1.cstToAst(e))) !== null && _b !== void 0 ? _b : []);
    return process(cstNode, primaryPrefixAst, primarySuffixAst);
}
exports.run = run;
function process(cstNode, prefix, suffix) {
    const methodInvocationSuffix = suffix.filter(e => e.kind === 'MethodInvocationSuffix');
    const arrayAccessSuffix = suffix.filter(e => e.kind === 'ArrayAccessSuffix');
    if (Array.isArray(arrayAccessSuffix) && arrayAccessSuffix.length > 0) {
        return createElementAccess(cstNode, prefix, suffix, arrayAccessSuffix);
    }
    if (Array.isArray(methodInvocationSuffix) && methodInvocationSuffix.length > 0) {
        return handleMethodInvocationSuffix(cstNode, prefix, suffix, methodInvocationSuffix);
    }
    return handleNoMethodInvocationSuffix(prefix, suffix);
}
/**
 * @param  {any} primaryPrefixAst
 * @param  {any} primarySuffixAst
 */
function handleNoMethodInvocationSuffix(primaryPrefixAst, primarySuffixAst) {
    if (primarySuffixAst.length === 1 && primarySuffixAst[0].kind === 'ClassLiteralSuffix') {
        primaryPrefixAst.push(...primarySuffixAst.pop().children);
    }
    if (primaryPrefixAst.length > 1) {
        return [
            toPropertyAccessExpression(primaryPrefixAst, false, []),
            ...primarySuffixAst
        ];
    }
    if (primaryPrefixAst.length === 1 && primaryPrefixAst[0].kind === 'ThisKeyword') {
        return [
            toPropertyAccessExpression([...primaryPrefixAst, ...primarySuffixAst], false, []),
        ];
    }
    if (primarySuffixAst.every(e => e.kind === 'Identifier') && primarySuffixAst.every(e => e.kind === 'Identifier')) {
        return [
            toPropertyAccessExpression([...primaryPrefixAst, ...primarySuffixAst], false, []),
        ];
    }
    return [
        ...primaryPrefixAst,
        ...primarySuffixAst
    ];
}
/**
 * @param  {any} cstNode
 * @param  {any} primaryPrefixAst
 * @param  {any} primarySuffixAst
 * @param  {any} methodInvocationSuffix
 */
function handleMethodInvocationSuffix(cstNode, primaryPrefixAst, primarySuffixAst, methodInvocationSuffix) {
    const identifierSuffix = primarySuffixAst.filter(e => e.kind === 'Identifier');
    const thisKeyword = primaryPrefixAst.find(e => e.kind === 'ThisKeyword');
    let obj = {
        kind: 'CallExpression',
        start: cstNode.location.startOffset,
        end: cstNode.location.endOffset,
        pos: cstNode.location.startOffset,
    };
    if (thisKeyword) {
        return getThisKeywordChildren(methodInvocationSuffix, thisKeyword, identifierSuffix, obj);
    }
    return getOtherCasesChildren(primaryPrefixAst, primarySuffixAst, methodInvocationSuffix, obj);
}
/**
 * @param  {any} methodInvocationSuffix
 * @param  {any} thisKeyword
 * @param  {any} identifierSuffix
 * @param  {any} obj
 * @returns any
 */
function getThisKeywordChildren(methodInvocationSuffix, thisKeyword, identifierSuffix, obj) {
    return {
        ...obj,
        children: [
            toPropertyAccessExpression([
                thisKeyword,
                ...identifierSuffix,
            ], true, genese_mapper_1.clone(methodInvocationSuffix)),
            ...getMethodInvocationSuffixChildren(methodInvocationSuffix),
        ]
    };
}
/**
 * @param  {any} primaryPrefixAst
 * @param  {any} primarySuffixAst
 * @param methodInvocationSuffix
 * @param  {any} obj
 * @returns any
 */
function getOtherCasesChildren(primaryPrefixAst, primarySuffixAst, methodInvocationSuffix, obj) {
    return {
        ...obj,
        children: [
            toPropertyAccessExpression([
                ...getNewExpression(primaryPrefixAst),
                ...primaryPrefixAst.filter(e => e.kind === 'Identifier'),
                ...primarySuffixAst.filter(e => e.kind === 'Identifier')
            ], true, genese_mapper_1.clone(methodInvocationSuffix)),
            ...primarySuffixAst.filter(e => e.kind === 'ClassLiteralSuffix'),
            ...getMethodInvocationSuffixChildren(methodInvocationSuffix),
        ]
    };
}
/** Get all MethodInvocationSuffix with children
 * @param  {} methodInvocationSuffixList
 */
function getMethodInvocationSuffixChildren(methodInvocationSuffixList) {
    let childrenList = [];
    if (Array.isArray(methodInvocationSuffixList)) {
        methodInvocationSuffixList.forEach(methodInvocationSuffix => {
            if (Array.isArray(methodInvocationSuffix.children) && methodInvocationSuffix.children.length > 0) {
                childrenList.push(...methodInvocationSuffix.children);
            }
        });
    }
    return childrenList;
}
/** Get newExpression Ast node
 * @param  {any} primaryPrefixAst
 * @returns any
 */
function getNewExpression(primaryPrefixAst) {
    const newExpression = primaryPrefixAst.filter(e => e.kind === 'NewExpression');
    if (Array.isArray(newExpression) && newExpression.length) {
        return [
            ...primaryPrefixAst.find(e => e.kind === 'NewExpression').children
        ];
    }
    return [];
}
/**
 * @param  {} primaryPrefixAst
 * @param primarySuffixAst
 * @param  {} arrayAccessSuffixList
 * @returns any
 */
function createElementAccess(cstNode, primaryPrefixAst, primarySuffixAst, arrayAccessSuffixList) {
    var _a, _b, _c;
    if (arrayAccessSuffixList.length === 1) {
        return singleAccessCase(cstNode, primaryPrefixAst, primarySuffixAst, arrayAccessSuffixList);
    }
    const arrayAccessSuffix = arrayAccessSuffixList.pop();
    const last = (_a = arrayAccessSuffix.children) === null || _a === void 0 ? void 0 : _a.find(e => e.kind === 'Identifier' || 'Literal');
    return {
        kind: 'ElementAccessExpression',
        start: (_b = primaryPrefixAst[0]) === null || _b === void 0 ? void 0 : _b.start,
        pos: (_c = primaryPrefixAst[0]) === null || _c === void 0 ? void 0 : _c.pos,
        end: arrayAccessSuffix === null || arrayAccessSuffix === void 0 ? void 0 : arrayAccessSuffix.end,
        children: [
            {
                ...createElementAccess(cstNode, primaryPrefixAst, primarySuffixAst, arrayAccessSuffixList),
            },
            last
        ]
    };
}
function singleAccessCase(cstNode, primaryPrefixAst, primarySuffixAst, arrayAccessSuffixList) {
    var _a, _b, _c;
    primarySuffixAst.pop();
    return {
        kind: 'ElementAccessExpression',
        start: (_a = primaryPrefixAst[0]) === null || _a === void 0 ? void 0 : _a.start,
        end: (_b = arrayAccessSuffixList[0]) === null || _b === void 0 ? void 0 : _b.end,
        pos: (_c = primaryPrefixAst[0]) === null || _c === void 0 ? void 0 : _c.pos,
        children: [
            ...[].concat(...[process(cstNode, primaryPrefixAst, primarySuffixAst)]),
            ...arrayAccessSuffixList[0].children.filter(e => e.kind === 'Identifier' || 'Literal')
        ]
    };
}
/**
 * @param  {any[]} identifiers
 * @param  {} isFunctionCall=false
 * @param  {any[]} methodInvocationSuffix
 * @returns any
 */
function toPropertyAccessExpression(identifiers, isFunctionCall = false, methodInvocationSuffix) {
    var _a, _b;
    if (!identifiers || identifiers.length === 0)
        return undefined;
    if (identifiers.length === 1 && methodInvocationSuffix.length === 0) {
        return identifiers[0];
    }
    if (identifiers.length === 1) {
        return { type: 'function', ...identifiers[0] };
    }
    const last = identifiers.pop();
    if (isFunctionCall) {
        return isFunctionCallCase(identifiers, last, methodInvocationSuffix);
    }
    return {
        kind: 'PropertyAccessExpression',
        start: (_a = identifiers[0]) === null || _a === void 0 ? void 0 : _a.start,
        end: last === null || last === void 0 ? void 0 : last.end,
        pos: (_b = identifiers[0]) === null || _b === void 0 ? void 0 : _b.pos,
        children: [
            toPropertyAccessExpression(identifiers, false, methodInvocationSuffix),
            last
        ].filter(e => e)
    };
}
function isFunctionCallCase(identifiers, last, methodInvocationSuffix) {
    var _a, _b;
    methodInvocationSuffix.pop();
    let start, pos;
    if (identifiers.length === 0) {
        start = last === null || last === void 0 ? void 0 : last.start;
        pos = last === null || last === void 0 ? void 0 : last.pos;
    }
    else {
        start = (_a = identifiers[0]) === null || _a === void 0 ? void 0 : _a.start;
        pos = (_b = identifiers[0]) === null || _b === void 0 ? void 0 : _b.pos;
    }
    return {
        kind: 'PropertyAccessExpression',
        start: start,
        end: last === null || last === void 0 ? void 0 : last.end,
        pos: pos,
        children: [
            toPropertyAccessExpression(identifiers, methodInvocationSuffix.length > 0, methodInvocationSuffix),
            { ...last, type: 'function' }
        ].filter(e => e)
    };
}
